@@ -9,6 +9,8 @@ import (
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/logging"
+	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-memdb"
 )
 
@@ -17,6 +19,8 @@ type Coordinate struct {
 	// srv is a pointer back to the server.
 	srv *Server
 
+	logger hclog.Logger
+
 	// updates holds pending coordinate updates for the given nodes. This is
 	// keyed by node:segment so we can get a coordinate for each segment for
 	// servers, and we only track the latest update per node:segment.
@@ -27,9 +31,10 @@ type Coordinate struct {
 }
 
 // NewCoordinate returns a new Coordinate endpoint.
-func NewCoordinate(srv *Server) *Coordinate {
+func NewCoordinate(srv *Server, logger hclog.Logger) *Coordinate {
 	c := &Coordinate{
 		srv:     srv,
+		logger:  logger.Named(logging.Coordinate),
 		updates: make(map[string]*structs.CoordinateUpdateRequest),
 	}
 
@@ -44,7 +49,7 @@ func (c *Coordinate) batchUpdate() {
 		select {
 		case <-time.After(c.srv.config.CoordinateUpdatePeriod):
 			if err := c.batchApplyUpdates(); err != nil {
-				c.srv.logger.Printf("[WARN] consul.coordinate: Batch update failed: %v", err)
+				c.logger.Warn("Batch update failed", "error", err)
 			}
 		case <-c.srv.shutdownCh:
 			return
@@ -66,7 +71,7 @@ func (c *Coordinate) batchApplyUpdates() error {
 	limit := c.srv.config.CoordinateUpdateBatchSize * c.srv.config.CoordinateUpdateMaxBatches
 	size := len(pending)
 	if size > limit {
-		c.srv.logger.Printf("[WARN] consul.coordinate: Discarded %d coordinate updates", size-limit)
+		c.logger.Warn("Discarded coordinate updates", "number_discarded", size-limit)
 		size = limit
 	}
 
@@ -134,12 +139,14 @@ func (c *Coordinate) Update(args *structs.CoordinateUpdateRequest, reply *struct
 	}
 
 	// Fetch the ACL token, if any, and enforce the node policy if enabled.
-	rule, err := c.srv.ResolveToken(args.Token)
+	authz, err := c.srv.ResolveToken(args.Token)
 	if err != nil {
 		return err
 	}
-	if rule != nil && c.srv.config.ACLEnforceVersion8 {
-		if !rule.NodeWrite(args.Node, nil) {
+	if authz != nil && c.srv.config.ACLEnforceVersion8 {
+		var authzContext acl.AuthorizerContext
+		structs.DefaultEnterpriseMeta().FillAuthzContext(&authzContext)
+		if authz.NodeWrite(args.Node, &authzContext) != acl.Allow {
 			return acl.ErrPermissionDenied
 		}
 	}
@@ -205,12 +212,15 @@ func (c *Coordinate) Node(args *structs.NodeSpecificRequest, reply *structs.Inde
 	}
 
 	// Fetch the ACL token, if any, and enforce the node policy if enabled.
-	rule, err := c.srv.ResolveToken(args.Token)
+
+	authz, err := c.srv.ResolveToken(args.Token)
 	if err != nil {
 		return err
 	}
-	if rule != nil && c.srv.config.ACLEnforceVersion8 {
-		if !rule.NodeRead(args.Node) {
+	if authz != nil && c.srv.config.ACLEnforceVersion8 {
+		var authzContext acl.AuthorizerContext
+		structs.WildcardEnterpriseMeta().FillAuthzContext(&authzContext)
+		if authz.NodeRead(args.Node, &authzContext) != acl.Allow {
 			return acl.ErrPermissionDenied
 		}
 	}
