@@ -1,10 +1,9 @@
 import Component from '@ember/component';
-import { get } from '@ember/object';
 export default Component.extend({
   tagName: 'fieldset',
   classNames: ['freetext-filter'],
   onchange: function(e) {
-    let searchable = get(this, 'searchable');
+    let searchable = this.searchable;
     if (!Array.isArray(searchable)) {
       searchable = [searchable];
     }
