@@ -1,6 +1,5 @@
 import RepositoryService from 'consul-ui/services/repository';
 import { inject as service } from '@ember/service';
-import { get } from '@ember/object';
 
 const modelName = 'session';
 export default RepositoryService.extend({
@@ -8,18 +7,19 @@ export default RepositoryService.extend({
   getModelName: function() {
     return modelName;
   },
-  findByNode: function(node, dc, configuration = {}) {
+  findByNode: function(node, dc, nspace, configuration = {}) {
     const query = {
       id: node,
       dc: dc,
+      ns: nspace,
     };
     if (typeof configuration.cursor !== 'undefined') {
       query.index = configuration.cursor;
     }
-    return get(this, 'store').query(this.getModelName(), query);
+    return this.store.query(this.getModelName(), query);
   },
   // TODO: Why Key? Probably should be findBySlug like the others
-  findByKey: function(slug, dc) {
-    return this.findBySlug(slug, dc);
+  findByKey: function(slug, dc, nspace) {
+    return this.findBySlug(...arguments);
   },
 });
